/*
 * Copyright (c) 2007-2010 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.sonatype.tycho.p2.publisher.cli;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.maven.Maven;
import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.execution.DefaultMavenExecutionRequest;
import org.apache.maven.execution.DefaultMavenExecutionResult;
import org.apache.maven.execution.MavenExecutionRequest;
import org.apache.maven.execution.MavenExecutionRequestPopulationException;
import org.apache.maven.execution.MavenExecutionRequestPopulator;
import org.apache.maven.execution.MavenExecutionResult;
import org.apache.maven.repository.RepositorySystem;
import org.apache.maven.settings.Settings;
import org.apache.maven.settings.building.DefaultSettingsBuildingRequest;
import org.apache.maven.settings.building.SettingsBuilder;
import org.apache.maven.settings.building.SettingsBuildingException;
import org.apache.maven.settings.building.SettingsBuildingRequest;
import org.codehaus.plexus.component.annotations.Component;
import org.codehaus.plexus.component.annotations.Requirement;
import org.codehaus.plexus.util.FileUtils;
import org.codehaus.plexus.util.IOUtil;

@Component( role = MavenEmbedder.class )
public class MavenEmbedder
{

    @Requirement
    private Maven maven;

    @Requirement
    private MavenExecutionRequestPopulator populator;

    @Requirement
    private SettingsBuilder settingsBuilder;

    @Requirement
    private RepositorySystem repositorySystem;

    private ArtifactRepository localRepository;

    private List<ArtifactRepository> remoteRepositories;

    private boolean offline;

    private boolean forceUpdate;

    private final File workDir;

    public MavenEmbedder()
    {
        workDir = FileUtils.createTempFile( "sra-", "", null );
        workDir.mkdirs();
        // TODO use some logger
        System.err.printf( "Temporary directory: %s\n", workDir.getAbsolutePath() );
    }

    public MavenExecutionResult execute( final InputStream pom,
                                         final File assemblyDir )
    {
        MavenExecutionResult result;
        OutputStream os = null;
        try
        {
            final File pomFile = new File( workDir, "pom.xml" );
            os = new BufferedOutputStream( new FileOutputStream( pomFile ) );
            IOUtil.copy( pom, os );
            os.close();

            final MavenExecutionRequest request = createExecutionRequest( pomFile );
            result = maven.execute( request );

            final File runtimeDir = new File( workDir, "target/tycho-p2/p2-install-folder" );
            if ( runtimeDir.exists() )
            {
                if ( !runtimeDir.renameTo( assemblyDir ) )
                {
                    // TODO this will not copy file permission
                    FileUtils.copyDirectoryStructure( runtimeDir, assemblyDir );
                }
            }
            else
            {
                result.addException( new Exception( "Runtime has not being build from unknown reasons" ) );
            }
        }
        catch ( final Exception e )
        {
            IOUtil.close( os );
            result = new DefaultMavenExecutionResult();
            result.addException( e );
        }
        return result;
    }

    public MavenExecutionRequest createExecutionRequest( final File pom )
        throws MavenExecutionRequestPopulationException
    {
        final MavenExecutionRequest request = new DefaultMavenExecutionRequest();

        populator.populateFromSettings( request, getSettings() );

        request.setLocalRepository( localRepository );
        request.setLocalRepositoryPath( localRepository.getBasedir() );
        request.setRemoteRepositories( remoteRepositories );
        request.setPluginArtifactRepositories( remoteRepositories );
        request.setOffline( offline );
        request.setUpdateSnapshots( forceUpdate );

        populator.populateDefaults( request );

        request.setPom( pom );
        request.setBaseDirectory( workDir );
        request.setGoals( Arrays.asList( "package" ) );

        request.setLoggingLevel( MavenExecutionRequest.LOGGING_LEVEL_DEBUG );
        request.setShowErrors( true );

        return request;
    }

    public Settings getSettings()
    {
        // MUST NOT use createRequest!

        // TODO: Can't that delegate to buildSettings()?
        final SettingsBuildingRequest request = new DefaultSettingsBuildingRequest();
        try
        {
            return settingsBuilder.build( request ).getEffectiveSettings();
        }
        catch ( final SettingsBuildingException ex )
        {
            // LOG ?
            final String msg = "Could not read settings.xml, assuming default values";
            return new Settings();
        }
    }

    public MavenEmbedder setLocalRepository( final String localRepository )
        throws Exception
    {
        if ( localRepository == null )
        {
            this.localRepository = repositorySystem.createDefaultLocalRepository();
        }
        else
        {
            this.localRepository = repositorySystem.createLocalRepository( new File( localRepository ) );
        }
        return this;
    }

    public MavenEmbedder setRemoteRepositories( final String[] remoteRepositories )
        throws Exception
    {
        if ( remoteRepositories == null || remoteRepositories.length == 0 )
        {
            this.remoteRepositories = Arrays.asList( repositorySystem.createDefaultRemoteRepository() );
        }
        else
        {
            this.remoteRepositories = new ArrayList<ArtifactRepository>();
            for ( final String remoteRepository : remoteRepositories )
            {
                this.remoteRepositories.add( repositorySystem.createArtifactRepository(
                    remoteRepository,
                    remoteRepository,
                    null,
                    null,
                    null ) );
            }
        }
        return this;
    }

    public MavenEmbedder setOffline( final boolean offline )
    {
        this.offline = offline;
        return this;
    }

    public MavenEmbedder setForceUpdate( final boolean forceUpdate )
    {
        this.forceUpdate = forceUpdate;
        return this;
    }

}
